<?php

namespace Drupal\payson_checkout;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class PaysonHandler implements PaysonHandlerInterface {
    
    public function __construct($paymentGatewayPlugin) {
      $this->paymentGatewayPlugin = $paymentGatewayPlugin;
    }
    
  /**
   * Creates an authorisation string .
   *
   * @param \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $paymentGatewayPlugin 
   *   Payment Gateway interface for the request.
   *
   * @return string
   *   The base64 encoded string used for authorisation.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function authorisationString(){
    if ( $this->paymentGatewayPlugin->getConfiguration()['mode'] == 'live' ) {
      return base64_encode($this->paymentGatewayPlugin->getConfiguration()['agent_id'] . ':' . $this->paymentGatewayPlugin->getConfiguration()['api_key']);
    } else {
      return base64_encode($this->paymentGatewayPlugin->getConfiguration()['agent_id_sand'] . ':' . $this->paymentGatewayPlugin->getConfiguration()['api_key_sand']);
    }
  }

/*
 * Get correct url for API calls depending on mode. 
 * 
 * $param $mode
 *   Get URL for this payment mode, if left empty use from configuration.
 * 
 */
  public function apiUrl($mode = null) {
    if (empty($mode) ) {
        // use current setttings
        $mode =$this->paymentGatewayPlugin->getConfiguration()['mode'];
    }
    if ( $mode == 'live' ) {
      return 'https://api.payson.se/2.0/';
    } else {
      return 'https://test-api.payson.se/2.0/';
    }
  }
 
  public function connectPayson($postData) {
    $httpClient = \Drupal::httpClient();

    $checkout = $httpClient->request('POST', $this->apiUrl() . 'Checkouts', [
      'headers' => [
        'Authorization' => 'Basic ' . $this->authorisationString(),
        'Content-Type' =>  'application/json',
      ],
      'body' => json_encode($postData),
    ]);

    if ($checkout->getStatusCode() == '201') {
      // success
      $body = $checkout->getBody()->getContents();
      $paysonCheckoutObj = json_decode($body, false);
      if ($paysonCheckoutObj) {
        return $paysonCheckoutObj;
      }
    }
    return false;
  }

  public function getCheckout(string $remoteId) {
    $httpClient = \Drupal::httpClient();

    try {
      $checkout = $httpClient->request('GET', $this->apiUrl() . 'Checkouts/' . $remoteId, [
        'headers' => [
          'Authorization' => 'Basic ' . $this->authorisationString(),
          'Content-Type' =>  'application/json',
        ],
      ]);
    } catch (\Exception $e) {
      if ( $e->getResponse()->getStatusCode() ) {
          return false;
      }
    }

    if ($checkout->getStatusCode() == '200') {
      // success
      $body = $checkout->getBody()->getContents();
      $paysonCheckoutObj = json_decode($body, false);
      if ($paysonCheckoutObj) {
          return $paysonCheckoutObj;
      }
    }
    return false;
  }

}
